using System;
using System.Collections.Generic;
using Jypeli;
using Jypeli.Assets;
using Jypeli.Controls;
using Jypeli.Widgets;

namespace pomppu_possu;

/// @author suvitoivanen
/// @version 14.02.2025
/// <summary>
/// Eeppinen tasohyppely peli
/// </summary>
public class PomppuPossu : PhysicsGame
{
    private const double NOPEUS = 200;
    private const double HYPPYNOPEUS = 750;
    private const int RUUDUN_KOKO = 60;

    private PlatformCharacter pomppuPossu;
    private Image _pomppuPossunKuva = LoadImage("pomppupossu.png");
    private Image _porkkanaKuva = LoadImage("porkkananKuva3.png");

    private SoundEffect _pisteAani = LoadSoundEffect("maali.wav");
/// <summary>
/// Pääohjelma luo kentän ja näppäinkomennot.
/// </summary>
    public override void Begin()
    {
        Gravity = new Vector(0, -1000);

        LuoKentta();
        
        Camera.Follow(pomppuPossu);
        Camera.ZoomFactor = 1.2;
        Camera.StayInLevel = true;
        MasterVolume = 0.5;
        Level.CreateBorders();

        Keyboard.Listen(Key.Up, ButtonState.Pressed, Hyppaa, "Pelaaja hyppää", pomppuPossu, HYPPYNOPEUS);
        Keyboard.Listen(Key.Left, ButtonState.Down, Liikuta, "Liikuttaa pelaajaa vasemmalle", pomppuPossu, -NOPEUS);
        Keyboard.Listen(Key.Right, ButtonState.Down, Liikuta, "Liikuttaa pelaajaa oikealle", pomppuPossu, NOPEUS);
        
        Keyboard.Listen(Key.Escape, ButtonState.Pressed, ConfirmExit, "Lopeta peli");
    }
    /// <summary>
    /// Funktio luo kentän hyödyntäen txt tiedostoa.
    /// </summary>
    private void LuoKentta()
    {
        TileMap kentta = TileMap.FromLevelAsset("kentta1.txt");
        kentta.SetTileMethod('#', LisaaTaso);
        kentta.SetTileMethod('N', LisaaPelaaja);
        kentta.SetTileMethod('*', LisaaPorkkana);
        kentta.Execute(RUUDUN_KOKO, RUUDUN_KOKO);
        Level.Background.CreateGradient(Color.White, Color.SkyBlue);
    }
   /// <summary>
   /// Funktio lisää kerättävät objektit kentälle
   /// </summary>
   /// <param name="paikka">Mihin objekti tulee kentällä</param>
   /// <param name="leveys">Leveys</param>
   /// <param name="korkeus">Korkeus</param>
    private void LisaaPorkkana(Vector paikka, double leveys, double korkeus)
    {
        Megaporkkanat megaporkkana = new Megaporkkanat(60, 60);
        megaporkkana.Position = paikka;
        megaporkkana.MakeStatic();
        megaporkkana.Image = _porkkanaKuva;
        Add(megaporkkana);
    }
/// <summary>
/// Funktio lisää tasot kentälle.
/// </summary>
/// <param name="paikka">paikka</param>
/// <param name="leveys">leveys</param>
/// <param name="korkeus">korkeus</param>
    private void LisaaTaso(Vector paikka, double leveys, double korkeus)
    {
        Tasot taso = new Tasot(leveys, 0.5*korkeus);
        taso.Position = paikka;
        taso.Color = Color.SpringGreen;
        taso.MakeStatic();
        Add(taso);
    }
/// <summary>
/// Funktio lisää pelaajan kentälle.
/// </summary>
/// <param name="paikka">paikka</param>
/// <param name="leveys">leveys</param>
/// <param name="korkeus">korkeus</param>
    private void LisaaPelaaja(Vector paikka, double leveys, double korkeus)
    {
        pomppuPossu = new PlatformCharacter(100, 100);
        pomppuPossu.Position = paikka;
        pomppuPossu.Mass = 4.0;
        pomppuPossu.Image = _pomppuPossunKuva;
        AddCollisionHandler<PhysicsObject, Megaporkkanat>(pomppuPossu, TormattiinEsineeseen);
        Add(pomppuPossu);
    }
/// <summary>
/// Funktio luo liikkeen sivuille pelaajalle, jota kutsutaan näppäimistöllä.
/// </summary>
/// <param name="pomppuPossu">pelaaja</param>
/// <param name="nopeus">liike</param>
    private void Liikuta(PlatformCharacter pomppuPossu, double nopeus)
    {
        pomppuPossu.Walk(nopeus);
    }
/// <summary>
/// Funktio luo liikkeen ylöspäin, jota kutsutaan näppäimistöllä.
/// </summary>
/// <param name="pomppuPossu">pelaaja</param>
/// <param name="nopeus">liike</param>
    private void Hyppaa(PlatformCharacter pomppuPossu, double nopeus)
    {
        pomppuPossu.Jump(nopeus);
    }
/// <summary>
/// Funktio käsittelee törmäykset.
/// </summary>
/// <param name="tormaaja">pelaaja</param>
/// <param name="kohde">kerättävät pisteet</param>
    public void TormattiinEsineeseen(PhysicsObject tormaaja, Megaporkkanat kohde)
    {
        kohde.OtaVastaanOsuma();
        _pisteAani.Play();
        MessageDisplay.Add("Megaporkkana kerätty!"); //Lisäsin viestin
    }

    /// <summary>
    /// Liikuttaa fysiikkaoliota.
    /// </summary>
    /// <param name="liikuteltava">Liikutettava</param>
    /// <param name="suunta">Suunta</param>
    public void Liikuta(PhysicsObject liikuteltava, Vector suunta)
    {
        liikuteltava.Push(suunta);
    }
        
}


